/* FILE NAME: getbind.c */
/* Get binding from name service database. */
#include <stdio.h>
#include "inv.h"
#include "..\status.h"

void do_import_binding(entry_name, binding_h)
char                  entry_name[];       /* entry name to begin search     */
rpc_binding_handle_t  *binding_h;         /* a binding handle               */
{
   unsigned long   status;                /* error status                   */
   RPC_NS_HANDLE   import_context;        /* required to import             */
   char            protseq[20];           /* protocol sequence              */

   status =
   RpcNsBindingImportBegin(      /* set context to import binding handles   */
      RPC_C_NS_SYNTAX_DEFAULT,       /* use default syntax                  */
      (unsigned char *)entry_name,   /* begin search with this name         */
      inv_v1_0_c_ifspec,          /* interface specification (inv.h)        */
      NULL,                          /* no optional object UUID required    */
      &import_context                /* import context obtained             */
   );
   CHECK_STATUS(status, "Can't begin import:", RESUME);

   while(1) {
      status =
      RpcNsBindingImportNext(                  /* import a binding handle   */
         import_context,        /* context from rpc_ns_binding_import_begin */
         binding_h              /* a binding handle is obtained             */
      );
      if(status != RPC_S_OK) {
         CHECK_STATUS(status, "Can't import a binding handle:", RESUME);
         break;
      }

      /** application specific selection criteria (by protocol sequence) *  */
      do_interpret_binding(*binding_h ,protseq);
      if(strcmp(protseq, "ncacn_ip_tcp") == 0)  /*select connection protocol*/
         break;
      else {
         status =
         RpcBindingFree(         /* free binding information not selected   */
            binding_h
         );
         CHECK_STATUS(status, "Can't free binding information:", RESUME);
      }
   } /*end while */

   status =
   RpcNsBindingImportDone(                    /* done with import context   */
      &import_context          /* obtained from rpc_ns_binding_import_begin */
   );
   return;
}
