/* FILE NAME: invntry.c */
/* A sample implementation of an inventory.                                 */
/* For simplicity, a few inventory items are maintained in the inventory.   */
/* The valid numbers are printed when the open_inventory() procedure is     */
/* called so the user knows what numbers to test.                           */
#include <stdio.h>
#include <stdlib.h>
#include "inv.h"
#define MAX_PARTS    10        /* maximum number of parts in this inventory */
#define MAX_SUBPARTS 5         /* maximum number of subparts for a part     */

static part_record *rec[MAX_PARTS]; /* array of pointers for this inventory */
static inventory_is_open = 0;       /* flag is reset to non-zero when open  */

/* Data for empty record or unknown part number */
static part_record no_part = {0,"UNKNOWN"};
static part_num    no_subparts[MAX_SUBPARTS];


void open_inventory()  /***** setup inventory *******************************/
{
   int i,j;
   unsigned size;

   /* Allocate memory for the inventory array.  Each part gets the size of  */
   /* a part_record plus enough memory for a subpart list.  Since the       */
   /* subpart list is already defined in the part_record as an array of 1,  */
   /* the new array memory only needs to be MAX_SUBPARTS-1 in size.       */
   for(i = 0; i < MAX_PARTS; i++) {
      size = sizeof(part_record) + (sizeof(part_num) * (MAX_SUBPARTS-1));
      rec[i] = (part_record *)malloc(size);
   }
   /* assign some data to the inventory array (part of an exercise machine) */
   rec[0]->number                = 102;
   strncpy((char *)rec[0]->name, "electronics display module", MAX_STRING);
   rec[0]->description = (paragraph)malloc(1000);
   strcpy((char *)rec[0]->description,
      "The electronics display module is a liquid crystal display containing\n\
a timer, counter, metronome, and calorie counter.");
   rec[0]->price.units           = ITEM;
   rec[0]->price.per_unit        = 7.00;
   rec[0]->quantity.units        = rec[0]->price.units;
   rec[0]->quantity.total.number = 432;
   rec[0]->subparts.size         = 4;  /* cannot be greater than MAX_SUBPARTS */
   for(i = 0; i < rec[0]->subparts.size; i++) /* values used are not relevant */
      rec[0]->subparts.numbers[i] = rec[0]->number + 1 + i;

   rec[1]->number                = 203;
   strncpy((char *)rec[1]->name, "base assembly", MAX_STRING);
   rec[1]->description = (paragraph)malloc(1000);
   strcpy((char *)rec[1]->description,
      "The base assembly rests on the floor to stabilize the machine.\n\
The arm and bench assemblies are attached to it.");
   rec[1]->price.units           = ITEM;
   rec[1]->price.per_unit        = 85.00;
   rec[1]->quantity.units        = rec[1]->price.units;
   rec[1]->quantity.total.number = 1078;
   rec[1]->subparts.size         = 5;  /* cannot be greater than MAX_SUBPARTS */
   for(i = 0; i < rec[1]->subparts.size; i++) /* values used are not relevant */
      rec[1]->subparts.numbers[i] = rec[1]->number + 17 + i;

   rec[2]->number                = 444;
   strncpy((char *)rec[2]->name, "ballast", MAX_STRING);
   rec[2]->description = (paragraph)malloc(1000);
   strcpy((char *)rec[2]->description,
      "The ballast is used to counter-balance the force exerted by the user.");
   rec[2]->price.units           = KILOGRAM;
   rec[2]->price.per_unit        = 1.59;
   rec[2]->quantity.units        = rec[2]->price.units;
   rec[2]->quantity.total.weight = 13456.2;
   rec[2]->subparts.size         = 0;  /* cannot be greater than MAX_SUBPARTS */
   for(i = 0; i < MAX_SUBPARTS; i++)  /* zero out subpart array */
      rec[2]->subparts.numbers[i] = no_subparts[i];

   /* fill in rest of inventory as "empty" data */
   for(i = 3; i < MAX_PARTS; i++) {
      rec[i] = &no_part;
      for(j = 0; j < MAX_SUBPARTS; j++)
         rec[i]->subparts.numbers[j] = no_subparts[j];
   }
   puts("Part numbers in inventory:");
   for(i = 0; i < MAX_PARTS; i++)
      if(rec[i]->number > 0)
         printf("%ld\n", rec[i]->number);
   inventory_is_open = 1;
   return;
}


void close_inventory()  /**** close inventory *******************************/
{
   /* Undo whatever is done in open_inventory.  Free memory and so forth.   */
   /* (not implemented) */
   return;
}


int read_part_record(number, part_ptr) /** get record for this part number **/
part_num number;
part_record **part_ptr;
{
   int i;

   if(inventory_is_open == 0)
      open_inventory();
   *part_ptr = &no_part;                      /* initialize assuming no part */
   for(i = 0; i < MAX_PARTS; i++)             /* search the inventory        */
      if(rec[i]->number == number) {          /* found the part              */
         *part_ptr = rec[i];
         break;
      }
   if( (*part_ptr)->number > 0)
      return(1);
   else                                       /* not a valid part            */
      return(0);
}


int write_part_record(part)  /*** update inventory for this part number *****/
part_record *part;
{
   int i;

   if(inventory_is_open == 0)
      open_inventory();
   for(i = 0; i < MAX_PARTS; i++)
      if(rec[i]->number == part->number) {
         rec[i] = part;   /* overwrite inventory with new data */
         return(1);
      }
   return(0);
}

/* dump the part data to the screen.
static dump_part_record(index)
int index;
{
   printf("number input:%ld  part number:%ld\n", number, rec[index]->number);
   printf("part name:%s\n", rec[index]->name);
   printf("description:%s\n", rec[index]->description);
   printf("price:%f per %s\n", rec[index]->price.per_unit, (rec[index]->price.units == ITEM) ? "item" : "gram");
   printf("quantity:");
   switch(rec[index]->quantity.units) {
   case ITEM: printf("%ld items\n", rec[index]->quantity.total.number); break;
   case GRAM: printf("%f grams\n", rec[index]->quantity.total.weight); break;
   case KILOGRAM: printf("%f kilos\n", rec[index]->quantity.total.weight); break;
   }
   printf("subparts: ");
   for(i = 0; i < rec[index]->subparts.size; i++)
      printf("%ld  ", rec[index]->subparts.numbers[i]);
   printf("\n");
}*/
