/* FILE NAME: manager.c */
/** Implementation of the remote procedures for the inventory application. **/
#include <stdio.h>
#include <stdlib.h>
#include "inv.h"

boolean is_part_available(number)
part_num number;
{
   part_record *part;                   /* a pointer to a part record */
   int found;

   found = read_part_record(number, &part);
   if(found)
      return(TRUE);
   else
      return(FALSE);
}


void whatis_part_name(number, name)
part_num  number;
part_name name;
{
   part_record *part;                   /* a pointer to a part record */

   read_part_record(number, &part);
   strncpy((char *)name, (char *)part->name, MAX_STRING);
   return;
}


paragraph get_part_description(number)
part_num  number;
{
   part_record *part;                   /* a pointer to a part record */
   paragraph description;
   int size;

   if( read_part_record(number, &part) ) {
      /* Allocated data that is returned to the client must be allocated */
      /* with the MIDL_user_allocate stub support routine.               */
      size = strlen((char *)part->description) + 1;
      description = (paragraph)MIDL_user_allocate((unsigned)size);
      strcpy((char *)description, (char *)part->description);
   }
   else
      description = NULL;
   return(description);
}


void whatis_part_price(number, price)
part_num   number;
part_price  *price;
{
   part_record *part;                   /* a pointer to a part record */

   read_part_record(number, &part);
   price->units = part->price.units;
   price->per_unit = part->price.per_unit;
   return;
}


void whatis_part_quantity(number, quantity)
part_num      number;
part_quantity *quantity;
{
   part_record *part;                   /* a pointer to a part record */

   read_part_record(number, &part);
   quantity->units = part->quantity.units;
   switch(quantity->units) {
      case ITEM: quantity->total.number = part->quantity.total.number;
                 break;
      case KILOGRAM:
      case GRAM: quantity->total.weight = part->quantity.total.weight;
                 break;
   }
   return;
}


void whatare_subparts(number, subpart_ptr)
part_num  number;
part_list **subpart_ptr;
{
   part_record *part;                           /* pointer to a part record */
   int i;
   int size;

   read_part_record(number, &part);

   /* Allocated data that is output to the client must be allocated with    */
   /* the MIDL_user_allocate stub support routine.  Allocate for a          */
   /* part_list struct plus the array of subpart numbers.  Remember the     */
   /* part_list struct already has an array with one element, hence the -1. */
   size = sizeof(part_list) + (sizeof(part_num) * (part->subparts.size-1));
   *subpart_ptr = (part_list *)MIDL_user_allocate((unsigned)size);

   /* fill in the values */
   (*subpart_ptr)->size = part->subparts.size;
   for(i = 0; i < (*subpart_ptr)->size; i++)
      (*subpart_ptr)->numbers[i] = part->subparts.numbers[i];
   return;
}


long int order_part(number, quantity, account)
part_num      number;
part_quantity *quantity;
account_num   account;
{
   part_record *part;           /* pointer to a part record */

   long error = 1;  /* assume no error to start */
   /* Test for valid input */
   if( !read_part_record(number, &part) ) /* invalid part number input */
      error = -1;
   else if(quantity->units == ITEM)       /* invalid quantity input    */
      error = (quantity->total.number <= 0) ? -2 : error;
   else if(quantity->units == GRAM || quantity->units == KILOGRAM)
      error = (quantity->total.weight <= 0.0) ? -2 : error;
   /* else if()   invalid account, not implemented */
   /*    error = -3;                               */
   if(error < 0)
      return(error);

   /* convert input quantity & units if units are not correct for part */
   if(quantity->units != part->quantity.units) {
      if(part->quantity.units == ITEM)     /* convert weight to items   */
         quantity->total.number = (long int)quantity->total.weight;
      else if(quantity->units == ITEM)     /* convert items to weight   */
         quantity->total.weight = (long float)quantity->total.number;
      else if(quantity->units == GRAM && part->quantity.units == KILOGRAM)
         quantity->total.weight /= 1000.0; /* convert grams to kilograms */
      else if(quantity->units == KILOGRAM && part->quantity.units == GRAM)
         quantity->total.weight *= 1000.0; /* convert kilograms to grams */
      quantity->units = part->quantity.units;
   }

   /* check if enough in inventory for this order */
   switch(part->quantity.units) {
   case ITEM:
      if(part->quantity.total.number > quantity->total.number)
         /* reduce quantity in inventory by amount ordered */
         part->quantity.total.number -= quantity->total.number;
      else {
         /* order all available and reduce quantity in inventory to 0 */
         quantity->total.number = part->quantity.total.number;
         part->quantity.total.number = 0;
      }
      break;
   case KILOGRAM:
   case GRAM:
      if(part->quantity.total.weight > quantity->total.weight)
         /* reduce quantity in inventory by amount ordered */
         part->quantity.total.weight -= quantity->total.weight;
      else {
         /* order all available and reduce quantity in inventory to 0.0 */
         quantity->total.weight = part->quantity.total.weight;
         part->quantity.total.weight = 0.0;
      }
      break;
   }

   write_part_record(part);    /* update inventory */

   return(1); /* order ok */
}
