/* FILE NAME: strbind.c */
/* Find a server binding handle from strings of binding information        */
/* including protocol sequence, host address, and server process endpoint. */
#include <stdio.h>
#include "rfile.h"
#include "status.h"                      /* contains the CHECK_STATUS macro */

int do_string_binding(host, binding_h) /*return=0 if binding valid, else -1 */
char             host[];     /* server host name or network address input   */
rpc_binding_handle_t *binding_h;    /* binding handle is output             */
{
   RPC_PROTSEQ_VECTOR   *protseq_vector;   /* protocol sequence list        */
   unsigned char        *string_binding;   /* string of binding information */
   unsigned long        status;            /* error status                  */
   int                  i, result;

   status =
   RpcNetworkInqProtseqs(    /* obtain a list of valid protocol sequences   */
      &protseq_vector                /* list of protocol sequences obtained */
   );
   CHECK_STATUS(status, "Can't get protocol sequences:", ABORT);

   /* loop through protocol sequences until a binding handle is obtained */
   for(i=0; i < protseq_vector->Count; i++) {

      status =
      RpcStringBindingCompose(     /* make string binding from components   */
         NULL,                           /* no object UUIDs are required    */
         protseq_vector->Protseq[i],     /* protocol sequence               */
         (unsigned char *)host,          /* host name or network address    */
         NULL,                           /* no endpoint is required         */
         NULL,                           /* no network options are required */
         &string_binding                 /* the constructed string binding  */
      );
      CHECK_STATUS(status, "Can't compose a string binding:", RESUME);

      status =
      RpcBindingFromStringBinding(    /* convert string to binding handle   */
         string_binding,                 /* input string binding            */
         binding_h                       /* binding handle is obtained here */
      );
      CHECK_STATUS(status, "Can't get binding handle from string:", RESUME);
      if(status != RPC_S_OK) {
         result = -1;
         CHECK_STATUS(status, "Can't get binding handle from string:", RESUME);
      }
      else
         result = 0;

      status =
      RpcStringFree(                       /* free string binding created   */
         &string_binding
      );
      CHECK_STATUS(status, "Can't free string binding:", RESUME);
      if(result == 0)  break;                        /* got a valid binding */
   }

   status =
   RpcProtseqVectorFree(           /* free the list of protocol sequences   */
      &protseq_vector
   );
   CHECK_STATUS(status, "Can't free protocol sequence vector:", RESUME);
   return(result);
}
